/*
	Pulse-width modulation (PWM) of 13 LEDs.
	
	
	The brightness (from 0 to PWMSTEPS-1 (refer to '../settings.h')) of each
	LED (stored in array 'led') is controlled via PWM. In order to
	decrease maximal power consumption (not averaged) this PWM consists of
	three cycles:
	
		Index	0   1   2   3   4   5   6   7   8   9  10  11  12
		Cycle   0   0   0   1   1   1   1   1   2   2   2   2   2
	
	All LEDs not belonging to the current cycle are switched off. That implies
	that all LEDs are driven at ~1/3 of their maximum brightness. The code
	below is very hardware dependend. It is ugly, consists of some kind of
	redundant parts but is fast.
	
	
	Author:         Gerrit Maus
	Date:         	2016-12-22
	Version:        1.0
	Changelog:      -
*/


#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include "../settings.h"
#include "led.h"


extern volatile uint8_t led[13];

volatile uint8_t pwmstep = 0;
volatile uint8_t cycle = 0;



void led_init(){
/*
	Initialises PWM.
*/

	DDRB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB6) | (1 << PB7);
	DDRD |= (1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3) | (1 << PD4) | (1 << PD5) | (1 << PD6) | (1 << PD7);
	
	TIMSK0 |= (1 << TOIE0);
	sei();
}

void led_enable(){

	
	/* Timer 0 overflow after 256 CPU cycles */
	TCCR0B |= (1 << CS00);
}

void led_disable(){

	
	/* Timer 0 stopped */
	TCCR0B &= ~(1 << CS00);
	
	/* Switch off all LEDs */
	PORTB &= ~((1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB6) | (1 << PB7));
	PORTD &= ~((1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3) | (1 << PD4) | (1 << PD5) | (1 << PD6) | (1 << PD7));
	
}


ISR(TIMER0_OVF_vect){
	
	int8_t i;
	
	/*
		Switch off LEDs depending on their choosen brightness and current
		PWM step
	*/
	
	switch(cycle){
		
		case 0:
		
			for(i = 0; i < 3; ++i){
				
				if(led[i] <= pwmstep){
			
					PORTB &= ~(1 << i);
				}
			}
		break;
		case 1:
			
			for(i = 3; i < 8; ++i){
				
				if(led[i] <= pwmstep){
					
					if(i < 5){
					
						PORTB &= ~(1 << (i + 3));
					}
					else{
					
						PORTD &= ~(1 << (i - 5));
					}
				}
			}
		break;
		default:
			
			for(i = 8; i < 13; ++i){
				
				if(led[i] <= pwmstep){
					
					PORTD &= ~(1 << (i - 5));
				}
			}
	}
	
	
	if(pwmstep == PWMSTEPS - 1){
		
		/* Next PWM cycle */
		pwmstep = 0;
		++cycle;
		
		if(cycle == 3){
			
			cycle = 0;
		}
		
		
		/* Switch on LEDs with choosen brightness > 0 */
		
		switch(cycle){
		
			case 0:
		
				for(i = 0; i < 3; ++i){
				
					if(led[i] != 0){
			
						PORTB |= (1 << i);
					}
				}
			break;
			case 1:
			
				for(i = 3; i < 8; ++i){
				
					if(led[i] != 0){
					
						if(i < 5){
					
							PORTB |= (1 << (i + 3));
						}
						else{
					
							PORTD |= (1 << (i - 5));
						}
					}
				}
			break;
			default:
			
				for(i = 8; i < 13; ++i){
				
					if(led[i] != 0){
					
						PORTD |= (1 << (i - 5));
					}
				}
		}
	}
	else{
		
		++pwmstep;
	}
}

